/*
 * PC Speaker beeper.
 * Copyright (c) 2006 Stanislav Lechev [AngelFire].
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * this program uses input device system of 2.6 kernels
 * so make sure to compile your kernel with:
 * CONFIG_INPUT=y
 * CONFIG_INPUT_MISC=y
 * CONFIG_INPUT_PCSPKR=y
 *
 * compile:
 * gcc beep.c speaker.c -o beep
 * or if you want some information on whats going on:
 * gcc -DPCSPKR_DEBUG beep.c -o beep
 *
 */ 

#include <stdio.h>
#include <unistd.h>
#include "speaker.h"


int main ( int argc, char ** argv ) {
	if (argc < 2) {
		printf("PC Speaker beeper.\n");
		printf("Copyright (c) 2006. Stanislav Lechev [AngelFire].\n");
		printf("You can use this program by the terms of GNU Public License v2.\n\n");
		printf("usage:\n\t    %s Khz/msecs [Khz/msecs]\n", argv[0]);
		printf("\tex: %s 500/100 750/100 1000/100\n", argv[0]);
		exit(1);
	}

	if (pcspkr_init() != 0) {
		fprintf(stderr, "Cannot find /dev/input/eventX device for pc speaker\n"
				"you need to compile your kernel with input device support and pc speaker as input device\n"
				"\tCONFIG_INPUT=y\n"
				"\tCONFIG_INPUT_MISC=y\n"
				"\tCONFIG_INPUT_PCSPKR=y\n");
		exit(1);
	}

	int i;
	for (i=1; i<argc; i++) {
		int khz,msec;
		if (sscanf(argv[i], "%d/%d", &khz, &msec) != 2) {
			fprintf(stderr, "unparsable argument [%d]: %s\n",
					i, argv[i]);
			pcspkr_tone(0, 0);
			exit(1);
		}
		
		pcspkr_tone(khz, msec);
	}

	pcspkr_tone(0, 0);
	return 0;
}

