/*
 * speaker.c:
 *
 * Copyright (c) 2006 Stanislav Lechev [AngelFire]. All rights reserved.
 *     <af@0xAF.org> [initial date 10.May.2006]
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * this program uses input device system of 2.6 kernels
 * so make sure to compile your kernel with:
 * CONFIG_INPUT=y
 * CONFIG_INPUT_MISC=y
 * CONFIG_INPUT_PCSPKR=y
 *
 */

#include <linux/input.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>

#include "speaker.h"

static const char rcsid[] = "$Id: speaker.c,v 1.2 2006/05/10 22:28:18 af Exp $";


#ifdef PCSPKR_DEBUG
#	ifdef PRINT_DEBUG
#		undef PRINT_DEBUG
#	endif
#	define PRINT_DEBUG(f, p...)	printf("DEBUG: " f, ## p);
#else
#	ifndef PRINT_DEBUG
#		define PRINT_DEBUG(...)
#	endif
#endif

int pcspkr_device_handle;

int pcspkr_init()
{
	int i;
	int version;
	unsigned short id[4];
	char name[256] = "Unknown";
	char devname[256];

	pcspkr_device_handle=-1;

	// try to find /dev/input/event? for pc speaker
	for (i=0; i<4; i++) {
		sprintf(devname, "/dev/input/event%d", i);
		PRINT_DEBUG("Opening device: %s\n", devname);

		pcspkr_device_handle = open(devname, O_RDWR|O_NONBLOCK);

		if (pcspkr_device_handle >= 0) {
			if (ioctl(pcspkr_device_handle, EVIOCGVERSION, &version)) {
				PRINT_DEBUG("evdev: can't get version\n");
				close(pcspkr_device_handle);
				pcspkr_device_handle = -1;
				continue;
			}
			PRINT_DEBUG("Input driver version is %d.%d.%d\n",
					version >> 16, (version >> 8) & 0xff, version & 0xff);

			ioctl(pcspkr_device_handle, EVIOCGID, id);
			PRINT_DEBUG("Input device ID: bus 0x%x vendor 0x%x product 0x%x version 0x%x\n",
					id[ID_BUS], id[ID_VENDOR], id[ID_PRODUCT], id[ID_VERSION]);

			ioctl(pcspkr_device_handle, EVIOCGNAME(sizeof(name)), name);
			PRINT_DEBUG("Input device name: \"%s\"\n", name);

			if (id[ID_BUS] != 0x10 || id[ID_VENDOR] != 0x1f || id[ID_PRODUCT] != 0x01 || id[ID_VERSION] != 0x100) {
				PRINT_DEBUG("not pc speaker\n");
				close(pcspkr_device_handle);
				pcspkr_device_handle = -1;
				continue;
			}

			break;

		} else {
			PRINT_DEBUG("open: %s\n", strerror(errno));
		}
	}

	if (!pcspkr_device_handle)
		return -1;

	return 0;
}

void pcspkr_tone(int tone, int delay)
{
	struct input_event event;

	PRINT_DEBUG("Sending tone: %d for %d msec\n", tone, delay);

	if (pcspkr_device_handle < 0) {
		PRINT_DEBUG("did you call pcspkr_init() ??\n");
		fprintf(stdout, "BEEP!\e[10;%d]\e[11;%d]\a\n", tone, delay);
		return;
	}

	memset(&event,0,sizeof(event));
	event.type=EV_SND;
	event.code=SND_TONE;

	event.value=tone;

	if (write(pcspkr_device_handle,&event,sizeof(event))!=sizeof(event)) {
		fprintf(stderr,"ERROR: cannot write to pc speaker device\n");
		return;
	}

	usleep(delay * 1000);

	event.value=0;
	if (write(pcspkr_device_handle,&event,sizeof(event))!=sizeof(event)) {
		fprintf(stderr,"ERROR: cannot write to pc speaker device\n");
		return;
	}

}

void pcspkr_finish()
{
	unsigned short id[4];

	if (pcspkr_device_handle >= 0) {
		if (ioctl(pcspkr_device_handle, EVIOCGID, id) ||
				id[ID_BUS] != 0x10 || id[ID_VENDOR] != 0x1f ||
				id[ID_PRODUCT] != 0x01 || id[ID_VERSION] != 0x100) {
			PRINT_DEBUG("not our pc speaker device.\n");
			pcspkr_device_handle = -1;
			return;
		}

		PRINT_DEBUG("closing pc speaker device\n");
		close(pcspkr_device_handle);
		pcspkr_device_handle = -1;
	}
}



#ifdef STANDALONE
int main()
{

	if (pcspkr_init()) {
		printf("error: init\n");
		exit(1);
	}

	pcspkr_beep();

	pcspkr_finish();

	return 0;
}
#endif
